@@ -77,8 +77,21 @@ KNOWN_CMSIS_DAP_IDS: List[VidPidPair] = [
     (VEGA_VID, 0x9527), # Vega VT-LinkII
     NXP_LPCLINK2_ID,
     NXP_MCULINK_ID,
+    (0x1a86, 0x8011),  # WCH-Link
+    (0x2a86, 0x8011),  # WCH-Link clone
     ]
 
+## List of substrings to look for in product and interface name strings.
+#
+# These strings identify a CMSIS-DAP compatible device. According to the specification,
+# "CMSIS-DAP" is required. But some low cost probes have misspelled or outright wrong
+# or missing strings.
+KNOWN_DEVICE_STRINGS: List[str] = (
+    "CMSIS-DAP",
+    "CMSIS_DAP",
+    "WCH-Link",
+    )
+
 ## List of VID/PID pairs for CMSIS-DAP probes that have multiple HID interfaces that must be
 # filtered by usage page. Currently these are only NXP probes.
 CMSIS_DAP_IDS_TO_FILTER_BY_USAGE_PAGE: List[VidPidPair] = [
@@ -90,6 +103,9 @@ def is_known_cmsis_dap_vid_pid(vid: int, pid: int) -> bool:
     """@brief Test whether a VID/PID pair belong to a known CMSIS-DAP device."""
     return (vid, pid) in KNOWN_CMSIS_DAP_IDS
 
+def is_known_device_string(device_string: str) -> bool:
+    return any(s in device_string for s in KNOWN_DEVICE_STRINGS)
+
 def filter_device_by_class(vid: int, pid: int, device_class: int) -> bool:
     """@brief Test whether the device should be ignored by comparing bDeviceClass.
 
